﻿function global:Set-VisualSPOnline
{
    [CmdletBinding()]
    Param
    (
        # The Site Collection URL of the site where VisualSP Online is to be installed.
        [Parameter(Mandatory = $true, ValueFromPipeline = $true, Position = 0)]
        [ValidateNotNullOrEmpty()]
        [string]$SiteCollectionUrl,

        # The Subscription ID assigned to you for VisualSP Online
        [Parameter(Mandatory = $false, Position = 1)]
        [ValidateNotNullOrEmpty()]
        [Guid]$SubscriptionId,

        # Include support for Modern Experience pages
        [Parameter(Position=2)]
        [Switch]$ModernExperience,

        # The path to the .sppkg file to deploy if supporting modern pages
        [Parameter(Position=3)]
        [ValidateNotNullOrEmpty()]
        [string]$SppkgPath,

        # Clear any previously cached credentials and prompt for new credentials.
        [Parameter(Position=4)]
        [Switch]$ClearTokenCache,

        [Parameter(Mandatory = $true, Position=5)]
        [ValidateSet("Online", "SP2013", "SP2016")]
        [string]$Target
    )

    Begin
    {
        if ($Target -eq "Online") {
            if ((Get-Module SharePointPnPPowerShellOnline -ListAvailable) -eq $null) {
                throw "The Microsoft SharePoint PnP PowerShell Online cmdlets have not been installed. See the following article for help with installation: https://docs.microsoft.com/en-us/powershell/sharepoint/sharepoint-pnp/sharepoint-pnp-cmdlets?view=sharepoint-ps"
            }
            Import-Module SharePointPnPPowerShellOnline -DisableNameChecking -Verbose:$false
        } elseif ($Target -eq "SP2013") {
            if ((Get-Module SharePointPnPPowerShell2013 -ListAvailable) -eq $null) {
                throw "The Microsoft SharePoint PnP PowerShell cmdlets for SharePoint 2013 have not been installed. See the following article for help with installation: https://docs.microsoft.com/en-us/powershell/sharepoint/sharepoint-pnp/sharepoint-pnp-cmdlets?view=sharepoint-ps"
            }
        } elseif ($Target -eq "SP2016") {
            if ((Get-Module SharePointPnPPowerShell2016 -ListAvailable) -eq $null) {
                throw "The Microsoft SharePoint PnP PowerShell cmdlets for SharePoint 2016 have not been installed. See the following article for help with installation: https://docs.microsoft.com/en-us/powershell/sharepoint/sharepoint-pnp/sharepoint-pnp-cmdlets?view=sharepoint-ps"
            }
        }
        if ($Target -ne "Online" -and $ModernExperience) {
            throw "The Modern Experience views are only supported in SharePoint Online."
        }
        if (![string]::IsNullOrEmpty($SppkgPath) -and !(Test-Path $SppkgPath)) {
            throw "Unable to locate the specified .sppkg file: $SppkgPath"
        }
    }
    Process
    {
        #region [Connect to the Site Collection]
        if ($Target -ne "Online") {
            if ($global:spCred -eq $null -or $ClearTokenCache) {
                $global:spCred = Get-Credential -Message "Enter your credentials for SharePoint:"
            }
        }
        $connErr = $null
        try {
            if ($Target -eq "Online") {
                $connection = Connect-PnPOnline -Url $SiteCollectionUrl -SPOManagementShell -ReturnConnection -ClearTokenCache:$ClearTokenCache -ErrorAction SilentlyContinue -ErrorVariable connErr
            } else {
                $connection = Connect-PnPOnline -Url $SiteCollectionUrl -Credentials $global:spCred -ReturnConnection -ErrorAction SilentlyContinue -ErrorVariable connErr
            }
        } catch {
            $connErr = $_.Exception.Message
            $connection = $null
        }
        if ($connErr -ne $null) {
            if ($connErr.Contains("(401) Unauthorized") -and $Target -eq "Online") {
                try {
                    $connection = Connect-PnPOnline -Url $SiteCollectionUrl -SPOManagementShell -ReturnConnection -ClearTokenCache
                } catch {
                    Write-Error $_
                    return
                }
            } else {
                Write-Error $connErr
                return
            }
        }
        if ($connection -eq $null) {
            return
        }
        #endregion

        $siteId = (Get-PnPSite -Includes Id).Id
        $userCustomActionName = "VisualSP Online Host Script"
        $userCustomActionExtensionName = "VisualSP Online Exension"

        $ucas = Get-PnPCustomAction -Scope Site

        $uca = $ucas | where { $_.Name -eq $userCustomActionName -or $_.Title -eq $userCustomActionName }
        if ($uca -ne $null) {
            Write-Host "Removing existing registration of VisualSP Online for Classic Experience Pages `"$SiteCollectionUrl`"..."
            Remove-PnPJavaScriptLink -Identity $uca -Scope Site -Confirm:$false
        }
		$url = "https://visualsponline.azurewebsites.net/app/js/VisualSPHost.min.js?Extension=None&SCCW=true&PowerShell=true&Version=1.0.11&SCID=$($siteId)"
        if ($SubscriptionId -ne [guid]::Empty -and $SubscriptionId -ne $null) {
            $url += "&SubscriptionId=$SubscriptionId"
        }
        Write-Host "Adding VisualSP Online for Classic Experience Pages `"$SiteCollectionUrl`"..."
        Add-PnPJavaScriptLink -Scope Site -Name $userCustomActionName -Sequence 0 -Url $url

        if ($ModernExperience -and $Target -eq "Online") {
            $app = Get-PnPApp | where {$_.Title -eq "VisualSP Online Application Extension"}
            if ($app -eq $null) {
                Write-Warning "The VisualSP Online Application Extension .sppkg is not installed in the app catalog."
                if ([string]::IsNullOrEmpty($SppkgPath)) {
                    Write-Warning "The path to the .sppkg file was not specified. You must rerun this script with the path specified or manually install the package."
                } else {
                    Write-Host "Adding the .sppkg file to the tenant application catalog..."
                    Add-PnPApp -Path $SppkgPath -SkipFeatureDeployment -Publish | Out-Null
                }
            }

            $ucae = $ucas | where { $_.Name -eq $userCustomActionExtensionName -or $_.Title -eq $userCustomActionExtensionName }
            if ($ucae -ne $null) {
                Write-Host "Removing existing registration of VisualSP Online for Modern Experience Pages at `"$SiteCollectionUrl`"..."
                Remove-PnPCustomAction -Identity $ucae -Scope Site -Confirm:$false
            }
            
            $csProps = "{`"subscriptionId`":`"`"}"
            if ($SubscriptionId -ne [guid]::Empty -and $SubscriptionId -ne $null) {
                $csProps = "{`"subscriptionId`":`"$SubscriptionId`"}"
            }

            Write-Host "Adding VisualSP Online for Modern Experience Pages at `"$SiteCollectionUrl`"..."
            Add-PnPCustomAction -Name $userCustomActionExtensionName `
                                -Title $userCustomActionExtensionName `
                                -Location "ClientSideExtension.ApplicationCustomizer" `
                                -ClientSideComponentId ([guid]"93ee580d-a111-4a21-bf47-d88e1b1e1cb1") `
                                -Scope Site `
                                -ClientSideComponentProperties $csProps

        }
    }
    End
    {
    }
}
